@@ -32,6 +32,7 @@ import textwrap
 # SOAP interface to Debian BTS
 import debianbts
 from collections import defaultdict
+from contextlib import suppress
 
 from . import checkversions
 
@@ -130,7 +131,25 @@ SEVLIST = ['critical', 'grave', 'serious', 'important', 'does-not-build',
 
 
 def convert_severity(severity, type='debbugs'):
-    "Convert severity names if needed."
+    """
+    Convert severity names if needed
+
+    Different bug tracking systems use different names for their
+    severity levels. This function does some translation between levels
+    known by 'debbugs' and 'gnats'.
+
+    Parameters
+    ----------
+    severity : str
+        bug severity level name
+    type : str
+        type of the bug tracking system (either 'debbugs' or 'gnats')
+
+    Returns
+    -------
+    str
+        translated severity level
+    """
     if type == 'debbugs':
         return {'non-critical': 'normal'}.get(severity, severity)
     elif type == 'gnats':
@@ -192,6 +211,29 @@ progenyother = {
 
 
 def check_package_info(package):
+    """
+    Get package information
+
+    Check if dpkg or apt know about a given package and return
+    information about it. If only a matching source package is found,
+    return information about one of its binary packages.
+
+    See :func:`utils.get_package_status()` for a description of the
+    returned package information.
+
+    This is a helper function used by :func:`handle_debian_ftp()`,
+    :func:`handle_debian_release()`, and :func:`handle_wnpp()`.
+
+    Parameters
+    ----------
+    package : str
+        package name
+
+    Returns
+    -------
+    (...) or None
+        Tuple with package information, or None of no package found
+    """
     # check dpkg database
     info = utils.get_package_status(package)
     if info[1]:
@@ -214,6 +256,47 @@ def check_package_info(package):
 
 
 def handle_debian_ftp(package, bts, ui, fromaddr, timeout, online=True, http_proxy=None):
+    """
+    Handle bugs for ftp.debian.org
+
+    Interactively (via the user interface) gather information required
+    for filing a bug on ftp.debian.org.
+
+    This function is accessible via:
+    SYSTEMS['debian']['specials']['ftp.debian.org']
+    and therefore one of the functions sharing the same call
+    signature available under:
+    SYSTEMS['debian']['specials']
+
+    Parameters
+    ----------
+    package : str
+        package name
+    bts
+        unused
+    ui : reportbug UI object
+        reportbug UI
+    fromaddr
+        unused
+    timeout
+        unused
+    online : bool, optional
+        unused
+    http_proxy : optional
+        unused
+
+    Returns
+    -------
+    (str, str, [str, ...], [], str, str) or None
+        Tuple with six components:
+
+        0. subject string
+        1. severity string
+        2. list of header strings
+        3. empty list of pseudoheaders
+        4. message body template
+        5. query bool, always False
+    """
     body = reason = archs = section = priority = ''
     suite = 'unstable'
     headers = []
@@ -393,6 +476,47 @@ def handle_debian_ftp(package, bts, ui, fromaddr, timeout, online=True, http_pro
 
 
 def handle_debian_release(package, bts, ui, fromaddr, timeout, online=True, http_proxy=None):
+    """
+    Handle bugs for release.debian.org
+
+    Interactively (via the user interface) gather information required
+    for filing a bug on release.debian.org.
+
+    This function is accessible via:
+    SYSTEMS['debian']['specials']['release.debian.org']
+    and therefore one of the functions sharing the same call
+    signature available under:
+    SYSTEMS['debian']['specials']
+
+    Parameters
+    ----------
+    package : str
+        package name
+    bts
+        unused
+    ui : reportbug UI object
+        reportbug UI
+    fromaddr
+        unused
+    timeout : int
+        remote query timeout in seconds
+    online : bool, optional
+        use a remote query to get the package version?
+    http_proxy : optional
+        unused
+
+    Returns
+    -------
+    (str, str, [], [str, ...], str, str) or None
+        Tuple with six components:
+
+        0. subject string
+        1. severity string, always 'normal'
+        2. empty list of headers
+        3. list of pseudoheaders strings
+        4. message body template
+        5. query bool, always False
+    """
     body = ''
     headers = []
     pseudos = []
@@ -449,24 +573,27 @@ def handle_debian_release(package, bts, ui, fromaddr, timeout, online=True, http
 
         query = False
 
-        if not info:
+        if info:
+            package = info[12] or package
+            version = info[0]
+
+        if online and tag.endswith('-pu'):
+            try:
+                version = list(checkversions.get_versions_available(package, timeout, (tag[:-3],)).values())[0]
+            except IndexError:
+                pass
+
+        if not version:
             cont = ui.select_options(
                 "This package doesn't appear to exist; continue?",
                 'yN', {'y': 'Ignore this problem and continue.',
                        'n': 'Exit without filing a report.'})
             if cont == 'n':
                 sys.exit(1)
-        else:
-            package = info[12] or package
 
     if tag in ('binnmu', 'unblock', stable_pu, oldstable_pu, 'rm'):
-        # FIXME: pu/rm should lookup the version elsewhere
-        version = info and info[0]
-        if online and tag.endswith('-pu'):
-            try:
-                version = list(checkversions.get_versions_available(package, timeout, (tag[:-3],)).values())[0]
-            except IndexError:
-                pass
+        if tag.endswith('-pu'):
+            version = None
         if version:
             cont = ui.select_options(
                 "Latest version seems to be %s, is this the proper one ?" % (version),
@@ -684,6 +811,47 @@ itp_template = textwrap.dedent("""\
 
 
 def handle_wnpp(package, bts, ui, fromaddr, timeout, online=True, http_proxy=None):
+    """
+    Handle wnpp bugs
+
+    Interactively (via the user interface) gather information required
+    for filing a wnpp (Work-Needing and Prospective Packages) bug.
+
+    This function is accessible via:
+    SYSTEMS['debian']['specials']['wnpp']
+    and therefore one of the functions sharing the same call
+    signature available under:
+    SYSTEMS['debian']['specials']
+
+    Parameters
+    ----------
+    package : str
+        package name
+    bts
+        unused
+    ui : reportbug UI object
+        reportbug UI
+    fromaddr : str
+        Mailaddress to use as Owner pseudo-header for ITP bugs
+    timeout : int
+        remote query timeout in seconds
+    online : bool, optional
+        use a remote query to get the package version?
+    http_proxy : str
+        Proxy URL to use with HTTP connections
+
+    Returns
+    -------
+    (str, str, [], [str, ...], str, str) or None
+        Tuple with six components:
+
+        0. subject string
+        1. severity string
+        2. list of header strings
+        3. list of pseudoheaders strings
+        4. message body template
+        5. query bool
+    """
     short_desc = body = ''
     headers = []
     pseudos = []
@@ -811,6 +979,47 @@ def handle_wnpp(package, bts, ui, fromaddr, timeout, online=True, http_proxy=Non
 
 
 def handle_installation_report(package, bts, ui, fromaddr, timeout, online=True, http_proxy=None):
+    """
+    Provide installation report template
+
+    Interactively (via the user interface) gather information required
+    for filing an installation report and provide a report template.
+
+    This function is accessible via:
+    SYSTEMS['debian']['specials']['installation-reports']
+    and therefore one of the functions sharing the same call
+    signature available under:
+    SYSTEMS['debian']['specials']
+
+    Parameters
+    ----------
+    package : str
+        unused
+    bts
+        unused
+    ui : reportbug UI object
+        reportbug UI
+    fromaddr : str
+        unused
+    timeout : int
+        unused
+    online : bool, optional
+        unused
+    http_proxy : str
+        unused
+
+    Returns
+    -------
+    (str, str, [], [str, ...], str, str) or None
+        Tuple with six components:
+
+        0. subject string
+        1. severity string
+        2. list of header strings
+        3. list of pseudoheaders strings
+        4. message body template
+        5. query bool
+    """
     body = ''
     headers = []
     pseudos = []
@@ -872,6 +1081,47 @@ def handle_installation_report(package, bts, ui, fromaddr, timeout, online=True,
 
 
 def handle_upgrade_report(package, bts, ui, fromaddr, timeout, online=True, http_proxy=None):
+    """
+    Provide upgrade report template
+
+    This function is not interactive and just provides a report
+    template.
+
+    This function is accessible via:
+    SYSTEMS['debian']['specials']['installation-reports']
+    and therefore one of the functions sharing the same call
+    signature available under:
+    SYSTEMS['debian']['specials']
+
+    Parameters
+    ----------
+    package : str
+        unused
+    bts
+        unused
+    ui : reportbug UI object
+        unused
+    fromaddr : str
+        unused
+    timeout : int
+        unused
+    online : bool, optional
+        unused
+    http_proxy : str
+        unused
+
+    Returns
+    -------
+    (str, str, [], [str, ...], str, str) or None
+        Tuple with six components:
+
+        0. subject string
+        1. severity string
+        2. list of header strings
+        3. list of pseudoheaders strings
+        4. message body template
+        5. query bool
+    """
     body = ''
     headers = []
     pseudos = []
@@ -920,6 +1170,17 @@ def handle_upgrade_report(package, bts, ui, fromaddr, timeout, online=True, http
 
 
 def dpkg_infofunc():
+    """
+    Collect information about dpkg and machine architecture(s)
+
+    This is a helper function for :func:`debian_infofunc()` and
+    :func:`ubuntu_infofunc()`.
+
+    Returns
+    -------
+    str
+        Information line(s) about debian/machine architecture(s)
+    """
     debarch = utils.get_arch()
     utsmachine = os.uname()[4]
     multiarch = utils.get_multiarch()
@@ -937,14 +1198,44 @@ def dpkg_infofunc():
 
 
 def debian_infofunc():
+    """
+    Get Debian release and Architecture information
+
+    This function is available under:
+    SYSTEMS['debian']['infofunc']
+
+    Returns
+    -------
+    str
+        Information line(s) about debian/machine architecture(s)
+    """
     return utils.get_debian_release_info() + dpkg_infofunc()
 
 
 def ubuntu_infofunc():
+    """
+    For Ubuntu get LSB release and Architecture information
+
+    This function is available under:
+    SYSTEMS['ubuntu']['infofunc']
+
+    Returns
+    -------
+    str
+        Information line(s) about LSB/machine architecture(s)
+    """
     return utils.lsb_release_info() + dpkg_infofunc()
 
 
 def generic_infofunc():
+    """
+    Get LSB release and Architecture information
+
+    Returns
+    -------
+    str
+        information about OS release and architecture
+    """
     utsmachine = os.uname()[4]
     return utils.lsb_release_info() + 'Architecture: %s\n\n' % utsmachine
 
@@ -1015,18 +1306,32 @@ TAGS = {
     'ipv6': 'This bug affects support for Internet Protocol version 6.',
     'lfs': 'This bug affects support for large files (over 2 gigabytes).',
     'l10n': 'This bug reports a localization/internationalization issue.',
-    'a11y': 'This bug is relevant to the accessibility of the package.',
+    'a11y': 'This bug affects accessibility for users with disabilities.',
     'newcomer': 'This bug has a known solution but the maintainer requests someone else implement it.',
     'ftbfs': 'The package fails to build from source.',
 }
 
 
 def get_tags(severity='', mode=utils.MODE_NOVICE):
-    """Returns the current tags list.
+    """
+    Return list of tags supported by the bug tracking system
 
-    If severity is release critical, than add additional tags not always present.
-    If mode is higher than STANDARD, then add suite tags."""
+    If severity is release critical or reportbug's user mode is higher
+    than STANDARD, then include the 'security' tag in the list of
+    available tags.
 
+    Parameters
+    ----------
+    severity : str
+        bug severity level name
+    mode : int
+        reportbug user mode level
+
+    Returns
+    -------
+    {str : str, ...}
+        dictionary of tag names and tag descriptions
+    """
     tags = TAGS.copy()
 
     if severity in ('grave', 'critical', 'serious'):
@@ -1042,6 +1347,24 @@ def get_tags(severity='', mode=utils.MODE_NOVICE):
 
 
 def yn_bool(setting):
+    """
+    Turn `setting` into equivalent string
+
+    If a yes/no setting comes as bool, convert it to a string.
+
+    This is a helper function for :func:`cgi_report_url()` and
+    :func:`cgi_package_url()`.
+
+    Parameters
+    ----------
+    setting : bool or str
+        indication whether the setting should be set or not
+
+    Returns
+    -------
+    str
+        'yes' or 'no'
+    """
     if setting:
         if str(setting) == 'no':
             return 'no'
@@ -1051,6 +1374,29 @@ def yn_bool(setting):
 
 
 def cgi_report_url(system, number, archived=False, mbox=False):
+    """
+    Construct bugreport URL string
+
+    Determine the URL of a bug report in the bug tracking system.
+
+    This function does the work for :func:`get_report_url()`.
+
+    Parameters
+    ----------
+    system : str
+        should be 'debian', 'ubuntu', or 'guug'
+    number : int
+        bug number
+    archived : bool, optional
+        consider archived bugs?
+    mbox : bool or str, optional
+        URL for bugreport website or mailbox?
+
+    Returns
+    -------
+    str or None
+        URL for the given bugreport
+    """
     root = SYSTEMS[system].get('cgiroot')
     if not root:
         return None
@@ -1059,6 +1405,31 @@ def cgi_report_url(system, number, archived=False, mbox=False):
 
 def cgi_package_url(system, package, archived=False, source=False,
                     repeatmerged=True, version=None):
+    """
+    Get the URL listing the bugs for a given package
+
+    This function does the work for :func:`get_package_url()`.
+
+    Parameters
+    ----------
+    system : str
+        should be 'debian', 'ubuntu', or 'guug'
+    package : str
+        package name
+    archived : bool, optional
+        specifies whether archived bugs should in included/listed
+    source : bool, optional
+        specifies whether archived bugs should in included/listed
+    repeatmerged : bool, optional
+        specifies whether merged bugs should be listed separately or not
+    version : str, optional
+        specifies a specific package version
+
+    Returns
+    -------
+    str
+        The URL of the package's bug page
+    """
     root = SYSTEMS[system].get('cgiroot')
     if not root:
         return None
@@ -1082,14 +1453,73 @@ def cgi_package_url(system, package, archived=False, source=False,
 
 def get_package_url(system, package, mirrors=None, source=False,
                     archived=False, repeatmerged=True):
+    """
+    Get the URL listing the bugs for a given package
+
+    Parameters
+    ----------
+    system : str
+        should be 'debian', 'ubuntu', or 'guug'
+    package : str
+        package name
+    mirrors : [str, ...], optional
+        list of mirror URLs
+    source : bool, optional
+        specifies whether archived bugs should in included/listed
+    archived : bool, optional
+        specifies whether archived bugs should in included/listed
+    repeatmerged : bool, optional
+        specifies whether merged bugs should be listed separately or not
+
+    Returns
+    -------
+    str
+        The URL of the package's bug page
+    """
     return cgi_package_url(system, package, archived, source, repeatmerged)
 
 
 def get_report_url(system, number, mirrors=None, archived=False, mbox=False):
+    """
+    Construct bugreport URL string
+
+    Determine the URL of a bug report in the bug tracking system.
+
+    Parameters
+    ----------
+    system : str
+        should be 'debian', 'ubuntu', or 'guug'
+    number : int
+        bug number
+    mirrors : [str, ...]
+        list of mirror URLs
+    archived : bool, optional
+        consider archived bugs?
+    mbox : bool or str, optional
+        URL for bugreport website or mailbox?
+
+    Returns
+    -------
+    str or None
+        URL for the given bugreport
+    """
     return cgi_report_url(system, number, archived, mbox)
 
 
 def parse_bts_url(url):
+    """
+    Parse bug tracking system URL
+
+    Parameters
+    ----------
+    url : str
+        the URL to parse
+
+    Returns
+    -------
+    (str, str)
+        A tuple with URL type (http, https, ...) and address
+    """
     bits = url.split(':', 1)
     if len(bits) != 2:
         return None
@@ -1104,14 +1534,15 @@ def parse_bts_url(url):
 
 # Dynamically add any additional systems found
 for origin in glob.glob('/etc/dpkg/origins/*'):
-    try:
-        fp = open(origin, errors='backslashreplace')
+    with suppress(OSError):
+        with open(origin, errors='backslashreplace') as fp:
+            lines = fp.readlines()
         system = os.path.basename(origin)
         SYSTEMS[system] = SYSTEMS.get(system, {'otherpkgs': {},
                                                'query-dpkg': True,
                                                'mirrors': {},
                                                'cgiroot': None})
-        for line in fp:
+        for line in lines:
             try:
                 (header, content) = line.split(': ', 1)
                 header = header.lower()
@@ -1132,13 +1563,60 @@ for origin in glob.glob('/etc/dpkg/origins/*'):
                         pass
             except ValueError:
                 pass
-        fp.close()
-    except IOError:
-        pass
 
 
 def get_reports(package, timeout, system='debian', mirrors=None, version=None,
                 http_proxy='', archived=False, source=False):
+    """
+    Collect bug reports for a given package
+
+    The purpose of collecting a list of reports is to allow the user to
+    check if the problem they are experiencing has already been
+    reported. However, there are often different (binary) packages to
+    which the same bug could be reported, and in many cases bugs are
+    also reported against source packages. It therefore makes sense to
+    return a relatively extensive list by default.
+
+    Reportbug's way of obtaining such an extensive list consists of (i)
+    collecting not only bugs reported against the package directly, but
+    also collecting bugs marked as affecting the package, and (ii) not
+    only returning bugs for a specified binary package, but also those
+    belonging to the corresponding source package. (The latter is done
+    in reportbug by calling this function with a list of packages
+    including the binary and its source.)
+
+    Parameters
+    ----------
+    package : str or list
+        name of affected package, or list of package names, or list of
+        bugs
+    timeout : int
+        connection timeout in seconds
+    system : str
+        should be either 'debian', 'ubuntu', or 'guug'. Only 'debian' is
+        currently supported.
+    mirrors
+        list of mirrors
+    version
+        package version
+    http_proxy : str
+        Proxy URL to use for HTTP connection
+    archived : bool
+        Also look up archived reports?
+    source : bool
+        Also look up source packages?
+
+    Returns
+    -------
+    (int, str, [(str, [debianbts.Bugreport, ...]), ...])
+        A tuple with three components:
+
+        0. the number of bugs,
+        1. title of this collection ("Bugs reports for XY"), and
+        2. hierarchy: list of two-tuples. Each tuple consists of a
+           string indicating the severity, and the list of
+           debianbts.Bugreport objects for that severity
+    """
     if system == 'debian':
         archived = 'both' if archived else '0'
         if isinstance(package, str):
@@ -1198,6 +1676,31 @@ def get_reports(package, timeout, system='debian', mirrors=None, version=None,
 
 def get_report(number, timeout, system='debian', mirrors=None,
                http_proxy='', archived=False, followups=False):
+    """
+    Look up a bug report in the bug tracking system
+
+    Parameters
+    ----------
+    number : int
+        bug number
+    timeout : int
+        connection timeout in seconds
+    system : str
+        should be either 'debian', 'ubuntu', or 'guug'
+    mirrors
+        list (or dictionary?) of mirrors
+    http_proxy : str
+        Proxy URL to use for HTTP connection
+    archived : bool
+        look for archived bugs, too?
+    followups : bool
+        passed on to called functions
+
+    Returns
+    -------
+    (debianbts.Bugreport, [str, ...])
+        Tuple with bugreport and its list of messages
+    """
     number = int(number)
 
     if system == 'debian':
