@@ -57,11 +57,29 @@ def ewrite(*args):
 
 
 def sign_message(body, fromaddr, package='x', pgp_addr=None, sign='gpg', draftpath=None):
-    '''Sign message with pgp key.'''
-    ''' Return: a signed body.
-        On failure, return None.
-        kw need to have the following keys
-    '''
+    """
+    Sign message with pgp key
+
+    Parameters
+    ----------
+    body : str
+        body of the message to be signed
+    fromaddr : str
+        email address of the sender. Ignored if pgp_addr is given.
+    package : str, optional
+        name of the package affected by the bug
+    pgp_addr : str, optional
+        email address identifying the ID with which to sign the message
+    sign : str, optional
+        if set to 'gpg', GnuPG is used for signing, otherwise PGP
+    draftpath : str, optional
+        directory to use for creating temporary files
+
+    Returns
+    -------
+    str
+        a signed message body, or None if signing did not work.
+    """
     if not pgp_addr:
         pgp_addr = get_email_addr(fromaddr)[1]
 
@@ -98,7 +116,7 @@ def sign_message(body, fromaddr, package='x', pgp_addr=None, sign='gpg', draftpa
         if not signedbody:
             raise NoMessage
         body = signedbody
-    except (NoMessage, IOError, OSError):
+    except (NoMessage, OSError):
         fh, tmpfile2 = TempFile(prefix=tempfile_prefix(package), dir=draftpath)
         fh.write(body)
         fh.close()
@@ -125,6 +143,26 @@ def _MIMEText_wrapper(text):
 
 
 def mime_attach(body, attachments, charset, body_charset=None):
+    """
+    Compose a Multipart MIME message from body text and attachments
+
+    Parameters
+    ----------
+    body : str
+        message body text
+    attachments : [str, ...]
+        list of files to attach
+    charset
+        ignored
+    body_charset
+        unused
+
+    Returns
+    -------
+    (email.Message, bool)
+        Tuple with MIME multipart message and a bool indicating whether
+        an error occurred when attaching files (False means no errors)
+    """
     mimetypes.init()
 
     message = MIMEMultipart('mixed')
@@ -163,32 +201,27 @@ def mime_attach(body, attachments, charset, body_charset=None):
         maintype, subtype = ctype.split('/', 1)
         if maintype == 'text':
             try:
-                with open(attachment, 'rU') as fp:
+                with open(attachment, 'r') as fp:
                     part = _MIMEText_wrapper(fp.read())
             except UnicodeDecodeError:
-                fp = open(attachment, 'rb')
                 part = MIMEBase(maintype, subtype)
-                part.set_payload(fp.read())
-                fp.close()
+                with open(attachment, 'rb') as fp:
+                    part.set_payload(fp.read())
                 email.encoders.encode_base64(part)
         elif maintype == 'message':
-            fp = open(attachment, 'rb')
-            part = MIMEMessage(email.message_from_binary_file(fp),
-                               _subtype=subtype)
-            fp.close()
+            with open(attachment, 'rb') as fp:
+                part = MIMEMessage(email.message_from_binary_file(fp),
+                                   _subtype=subtype)
         elif maintype == 'image':
-            fp = open(attachment, 'rb')
-            part = MIMEImage(fp.read(), _subtype=subtype)
-            fp.close()
+            with open(attachment, 'rb') as fp:
+                part = MIMEImage(fp.read(), _subtype=subtype)
         elif maintype == 'audio':
-            fp = open(attachment, 'rb')
-            part = MIMEAudio(fp.read(), _subtype=subtype)
-            fp.close()
+            with open(attachment, 'rb') as fp:
+                part = MIMEAudio(fp.read(), _subtype=subtype)
         else:
-            fp = open(attachment, 'rb')
             part = MIMEBase(maintype, subtype)
-            part.set_payload(fp.read())
-            fp.close()
+            with open(attachment, 'rb') as fp:
+                part.set_payload(fp.read())
             email.encoders.encode_base64(part)
         part.add_header('Content-Disposition', 'attachment',
                         filename=os.path.basename(attachment))
@@ -204,8 +237,84 @@ def send_report(body, attachments, mua, fromaddr, sendto, ccaddr, bccaddr,
                 smtptls=False, smtphost='localhost',
                 smtpuser=None, smtppasswd=None, paranoid=False, draftpath=None,
                 envelopefrom=None):
-    '''Send a report.'''
+    """Send a report
 
+    This function handles all variants supported by reportbug of
+    'sending' a report, including:
+     - passing to a local MUA (mutt, thunderbird, evolution etc.),
+     - submitting through a remote smarthost,
+     - passing to a local MTA (exim, sendmail etc.),
+     - saving to a file,
+     - piping the message to standard output.
+
+    Parameters
+    ----------
+    body : str
+        message body text
+    attachments : [str, ...]
+        list of files to attach
+    mua : reportbug.utils.Mua()
+        mail user agent to use for submitting the report
+    fromaddr : str
+        Mail address to use in the From: header
+    sendto : str
+        recipient address
+    ccaddr : str
+        mailaddress to put in CC: header
+    bccaddr
+        mailaddress to put in BCC: header
+    headers : [(str, str), ...]
+        list of (header, value) tuples
+    package : str, optional
+        name of the package the report is about
+    charset : str, optional
+        passed to mime_attach
+    mailing : bool, optional
+        if True, create Message-ID, X-Mailer and Date headers
+    sysinfo : dict, optional
+        dictionary with system info
+    rtype : str, optional
+        should be 'debbugs' for best results
+    exinfo : str or bool, optional
+        whether the message is extra info on an existing bug (followup)
+        or not
+    replyto : str, optional
+        mailaddress to put in Reply-To: header. If there is REPLYTO
+        environment variable, the value of the variable will be used
+        instead.
+    printonly : bool, optional
+        if True, do not send the report, just print it to standard
+        output
+    template : bool, optional
+        if True, do not send the report, just print it to standard
+        output (same effect as `printonly`)
+    outfile : str, optional
+        name of a file to which to write the message
+    mta : str, optional
+        path of a mail transfer agent to use
+    kudos : bool, optional
+        if True, indicates the message is meant as appreciation mail to
+        the maintainer rather than bug information
+    smtptls : bool, optional
+        if True, try to use TLS during submission to smtp host
+    smtphost : str, optional
+        host name of the SMTP host
+    smtpuser : str, optional
+        user name to use for authentication at the SMTP host
+    smtppasswd : str, optional
+        password to use for authentication at the SMTP host
+    paranoid : bool, optional
+        if True, view the report in a pager again and ask for
+        confirmation before finally submitting
+    draftpath : str, optional
+        directory where to store temporary files
+    envelopefrom : str, optional
+        sender address to use in envelope when using a local mta
+
+    Returns
+    -------
+    None
+    """
     failed = using_sendmail = False
     msgname = ''
     # Disable smtphost if mua is set
@@ -397,7 +506,7 @@ def send_report(body, attachments, mua, fromaddr, sendto, ccaddr, bccaddr,
         try:
             pipe.write(message)
             pipe.flush()
-        except IOError:
+        except OSError:
             failed = True
             pipe.close()
 
