@@ -25,12 +25,11 @@ import subprocess
 import re
 import math
 import string
-import errno
 import glob
 import getpass
 import textwrap
 import locale
-from functools import reduce
+from contextlib import suppress
 try:
     import readline
 except ImportError:
@@ -467,14 +466,9 @@ def show_report(number, system, mirrors,
             text = 'Original report - %s\n\n%s' % (buginfo.subject, messages[0])
 
         if not skip_pager:
-            try:
+            with suppress(BrokenPipeError):
                 with subprocess.Popen('sensible-pager', stdin=subprocess.PIPE, errors='ignore') as fd:
                     fd.communicate(text)
-            except IOError as x:
-                if x.errno == errno.EPIPE:
-                    pass
-                else:
-                    raise
         skip_pager = False
 
         options = 'xOrbeq'
@@ -789,7 +783,7 @@ def proc_hierarchy(hierarchy):
 
     lenlist = [len(i[1]) for i in hierarchy]
     if lenlist:
-        count = reduce(lambda x, y: x + y, lenlist)
+        count = sum(lenlist)
     else:
         return 0, 0
 
@@ -1012,11 +1006,9 @@ def display_report(text, use_pager=True, presubj=False):
         text += "\n(You may need to press 'q' to exit your pager and continue using\nreportbug at this point.)"
 
     pager = os.environ.get('PAGER', 'sensible-pager')
-    try:
+    with suppress(OSError):
         with subprocess.Popen(pager, stdin=subprocess.PIPE, errors='ignore') as p:
             p.communicate(text)
-    except IOError:
-        pass
 
 
 def spawn_editor(message, filename, editor, charset='utf-8'):
