@@ -101,6 +101,26 @@ _apt_cache = apt.Cache()
 
 
 def realpath(filename):
+    """
+    Return the real path of file `filename`
+
+    This is similar to ``os.path.realpath()``. The difference is that
+    symlinks to a small set of basic directories are not resolved and
+    just kept in the path if present.
+
+    This function is a helper function called only by
+    :func:`search_path_for()`.
+
+    Parameters
+    ----------
+    filename : string
+        name of a file, may include relative or absolute path
+
+    Returns
+    -------
+    str
+        the real path of the given file
+    """
     filename = os.path.abspath(filename)
 
     bits = filename.split('/')
@@ -124,6 +144,29 @@ pathdirs = ['/usr/sbin', '/usr/bin', '/sbin', '/bin', '/usr/X11R6/bin',
 
 
 def search_path_for(filename):
+    """
+    Searches for a file in PATH and common application directories
+
+    Given the filename of an executable, this function tries to locate
+    the file in common application directories (/usr/sbin, /usr/bin/,
+    /sbin, /bin, /usr/X11R6/bin, /usr/games) and in any additional
+    directories included in PATH.
+
+    :func:`realpath()` is called on the result (if found).
+
+    This function is a helper function called only by
+    :func:`find_package_for()`.
+
+    Parameters
+    ----------
+    filename : string
+        name of a file, may include relative or absolute path
+
+    Returns
+    -------
+    str or None
+        the real path of the given file; None if the file was not found
+    """
     d, f = os.path.split(filename)
     if d:
         return realpath(filename)
@@ -141,17 +184,20 @@ def search_path_for(filename):
 
 
 def which_editor(specified_editor=None):
-    """ Determine which editor program to use.
+    """
+    Determine which editor program to use
 
-        :parameters:
-          `specified_editor`
-            Specified editor for reportbug, to be used in preference
-            to other settings.
+    Parameters
+    ----------
+    specified_editor : str
+        Specified editor for reportbug, to be used in preference
+        to other settings.
 
-        :return value:
-            Command to invoke for selected editor program.
-
-        """
+    Returns
+    -------
+    str
+        Command to invoke for selected editor program.
+    """
     debian_default_editor = "/usr/bin/sensible-editor"
     for editor in [specified_editor,
                    os.environ.get("VISUAL"),
@@ -164,11 +210,50 @@ def which_editor(specified_editor=None):
 
 
 def glob_escape(filename):
+    """
+    Escape globbing pattern characters with backslash
+
+    This function is a helper function used only by
+    :func:`query_dpkg_for()`.
+
+    Parameters
+    ----------
+    filename : str
+        filename, possibly using a globbing pattern
+
+    Returns
+    -------
+    str
+        escaped filename (pattern)
+    """
     filename = re.sub(r'([*?\[\]])', r'\\\1', filename)
     return filename
 
 
 def search_pipe(searchfile, use_dlocate=True):
+    """
+    Search for a file in a debian binary package
+
+    This is done by opening a pipe to capture the output of ``dlocate``
+    or ``dpkg``.
+
+    This function is a helper function used only by
+    :func:`query_dpkg_for()`.
+
+    Parameters
+    ----------
+    searchfile : str
+        the file to look for
+    use_dlocate : bool
+        try to search using dlocate (which is fast, but may not be
+        available) or always use dpkg (always available, but slow)
+
+    Returns
+    -------
+    (pipe, bool)
+        Tuple with the pipe and a bool, the latter indicating whether
+        dlocate was used (True) or not (False)
+    """
     arg = shlex.quote(searchfile)
     if use_dlocate and os.path.exists('/usr/bin/dlocate'):
         pipe = os.popen('COLUMNS=79 dlocate -S %s 2>/dev/null' % arg)
@@ -179,6 +264,24 @@ def search_pipe(searchfile, use_dlocate=True):
 
 
 def get_command_output(cmd):
+    """
+    Capture the output of a program
+
+    Since reportbug needs to be able to run in misconfigured locales, we
+    need to specify an error handler whenever some external input is
+    processed.  This wrapper function avoids having to specify this
+    separately for each program invocation.
+
+    Parameters
+    ----------
+    cmd : str or tuple
+        The command to execute including any options and arguments
+
+    Returns
+    -------
+    str
+        The output (stdout) of the command
+    """
     use_shell = False
     if isinstance(cmd, str) and ' ' in cmd:
         use_shell = True
@@ -186,6 +289,29 @@ def get_command_output(cmd):
 
 
 def query_dpkg_for(filename, use_dlocate=True):
+    """
+    Search for a file in a debian binary package
+
+    This is done using ``dlocate`` or ``dpkg``.
+
+    This function is a helper function used only by
+    :func:`find_package_for()`.
+
+    Parameters
+    ----------
+    filename : str
+        the file to look for
+    use_dlocate : bool
+        try to search using dlocate (which is fast, but may not be
+        available) or always use dpkg (always available, but slow)
+
+    Returns
+    -------
+    (str, dict)
+        Tuple with the original filename input parameter and a
+        dictionary with matching packages as keys and lists of
+        corresponding files as values.
+    """
     try:
         _ = os.getcwd()
     except OSError:
@@ -224,8 +350,26 @@ def query_dpkg_for(filename, use_dlocate=True):
 
 
 def find_package_for(filename, pathonly=False):
-    """Find the package(s) containing this file."""
+    """
+    Find the package(s) containing this file
 
+    Parameters
+    ----------
+    filename : str
+        The name of the file including any path components
+    pathonly : bool
+        If no path component is part of the file name, specify if the
+        file should be looked for in the ``PATH`` (directories with
+        executables) only or if the entire dpkg database should be
+        searched.
+
+    Returns
+    -------
+    (str, dict)
+        Tuple with file name (possibly modified from input parameter
+        filename) and a dictionary with matching packages as keys and
+        lists of corresponding files as values.
+    """
     packages = {}
 
     # tries to match also files in /var/lib/dpkg/info/
@@ -248,12 +392,31 @@ def find_package_for(filename, pathonly=False):
 
 
 def find_rewritten(username):
+    """
+    Lookup email alias of user `username`
+
+    Checks if there is an email alias for the user `username` configured
+    for outgoing mail (in '/etc/email-addresses') and returns that
+    alias.
+
+    This is a helper function for :func:`get_user_id()`.
+
+    Parameters
+    ----------
+    username : str
+        the username to look up
+
+    Returns
+    -------
+    str or None
+        an email address (or None if not found)
+    """
     filename = '/etc/email-addresses'
     try:
         fp = open(filename, errors='backslashreplace')
-    except (FileNotFoundError, IOError):
+    except OSError:
         return None
-    try:
+    with fp:
         for line in fp:
             line = line.strip().split('#')[0]
             if not line:
@@ -265,12 +428,25 @@ def find_rewritten(username):
             except ValueError:
                 print('Invalid entry in %s' % filename)
                 return None
-    finally:
-        fp.close()
 
 
 def check_email_addr(addr):
-    """Simple check for email validity"""
+    """
+    Simple check for email validity
+
+    Performs some simple checks on whether the given string looks like
+    an email address.
+
+    Parameters
+    ----------
+    addr : str
+        string to check
+
+    Returns
+    -------
+    bool
+        True if the string looks like an email address, otherwise False
+    """
     if '@' not in addr:
         return False
     if addr.count('@') != 1:
@@ -293,14 +469,75 @@ def check_email_addr(addr):
 
 
 def get_email_addr(addr):
+    """
+    Extract email address from a string
+
+    Email addresses are often accompanied by a name: ``My Name
+    <my@domain.org>`` or ``my@address.info (My Name)``. This function
+    tries to separate the two parts and returns a (name, address) tuple.
+
+    Parameters
+    ----------
+    addr : str
+        an email address (possibly including a name component)
+
+    Returns
+    -------
+    (str, str)
+        Tuple containing the name (if found, otherwise empty string) and
+        the plain email address
+    """
     return email.utils.getaddresses([addr, ])[0]
 
 
 def get_email(emailaddr='', realname=''):
+    """
+    Get the user's email address and name
+
+    If name and email address are not given, try to obtain the missing
+    info for the current user and return the result.
+
+    Parameters
+    ----------
+    emailaddr : str
+        an email address (possibly including a name component)
+    realname : str
+        Real name (will be ignored if the `emailaddr` string contains a name
+        component)
+
+    Returns
+    -------
+    (str, str)
+        Tuple containing the name (if found, otherwise empty string) and
+        the plain email address
+    """
     return get_email_addr(get_user_id(emailaddr, realname))
 
 
 def get_user_id(emailaddr='', realname='', charset='utf-8'):
+    """
+    Find email address of the current user
+
+    This is similar to :func:`get_email()`, but returns a single string
+    instead of a (name, address) tuple.
+
+    Parameters
+    ----------
+    emailaddr : str
+        address, if already known, possibly including a name component.
+        Note that if the 'REPORTBUGEMAIL' environment variable is set,
+        its value will overwrite this parameter.
+    realname : str
+        real name, if already known.
+    charset : str
+        not used
+
+    Returns
+    -------
+    str
+        email address string, including a name component if available
+        (RFC2047-encoded if the name contains non-ASCII characters)
+    """
     uid = os.getuid()
     info = pwd.getpwuid(uid)
     emailaddr = (os.environ.get('REPORTBUGEMAIL', emailaddr)
@@ -313,7 +550,7 @@ def get_user_id(emailaddr='', realname='', charset='utf-8'):
         try:
             with open('/etc/mailname', 'r') as mf:
                 domainname = mf.readline().strip()
-        except (FileNotFoundError, IOError):
+        except OSError:
             domainname = socket.getfqdn()
 
         emailaddr = emailaddr + '@' + domainname
@@ -340,6 +577,44 @@ statuscache = {}
 
 
 def get_package_status(package, avail=False):
+    """
+    Get information on a binary package, including status
+
+    Parameters
+    ----------
+    package : str
+        the name of the package
+    avail : bool
+        if True, also look up the detailed package information if the
+        package is not installed (with apt rather than dpkg)
+
+    Returns
+    -------
+    (..., ...)
+        A tuple with 17 components. The components are:
+
+        0.  package version string or None
+        1.  available package name or None
+        2.  dependencies as a tuple containing lists, each list
+            representing alternative dependencies
+        3.  recommended packages as a tuple containing lists, each list
+            representing alternative recommends
+        4.  conffiles as a tuple containing (filename_str, md5sum_str)
+            tuples
+        5.  maintainer name and email address string
+        6.  bool indicating whether the package is installed
+        7.  origin (content of "Origin:" package header if available)
+        8.  vendor of the package if available
+        9.  reportinfo
+        10. priority string
+        11. package short description string (summary)
+        12. source name if a source header is available
+        13. full package description string (without summary line)
+        14. package state string
+        15. suggested packages as a tuple containing lists, each list
+            representing alternative suggests
+        16. archive section of the package
+    """
     if not avail and package in statuscache:
         return statuscache[package]
 
@@ -484,6 +759,12 @@ def get_package_status(package, avail=False):
 
 # Object that essentially chunkifies the output of apt-cache dumpavail
 class AvailDB(object):
+    """
+    Split a pipe or file into chunks separated by empty lines
+
+    This turns a pipe or file into an iterator that returns chunk by
+    chunk on each next() call.
+    """
     def __init__(self, fp=None, popenob=None):
         self.popenob = popenob
         if fp:
@@ -530,17 +811,50 @@ class AvailDB(object):
 
 
 def get_dpkg_database():
-    subp = subprocess.Popen(('dpkg-query', '--status'), errors="backslashreplace", stdout=subprocess.PIPE)
+    """
+    Get the dpkg status database as an :class:`AvailDB` object.
+
+    This is a helper function for :func:`get_package_info()`.
+
+    Returns
+    -------
+    AvailDB
+        A :class:`AvailDB` object to access the dpkg status database
+    """
+    subp = subprocess.Popen(('dpkg-query', '--status'), errors="backslashreplace", stdout=subprocess.PIPE, text=True)
     return AvailDB(popenob=subp)
 
 
 def get_avail_database():
+    """
+    Get the database of available packages as an :class:`AvailDB` object
+
+    This function is not used anywhere and might be removed soon.
+
+    Returns
+    -------
+    AvailDB
+        A :class:`AvailDB` object to access apt's available database
+    """
     # print >> sys.stderr, 'Searching available database'
-    subp = subprocess.Popen(('apt-cache', 'dumpavail'), stdout=subprocess.PIPE)
+    subp = subprocess.Popen(('apt-cache', 'dumpavail'), stdout=subprocess.PIPE, text=True)
     return AvailDB(popenob=subp)
 
 
 def get_source_name(package):
+    """
+    Get the source package name of a given package
+
+    Parameters
+    ----------
+    package : str
+        the name of a (source or binary) package
+
+    Returns
+    -------
+    str or None
+        the name of the corresponding source package
+    """
     try:
         return _apt_cache[package].versions[0].source_name
     except KeyError:
@@ -556,6 +870,19 @@ def get_source_name(package):
 
 
 def get_source_version(srcname):
+    """
+    Get the package version of a given source package
+
+    Parameters
+    ----------
+    package : str
+        the name of a source package
+
+    Returns
+    -------
+    str or None
+        the version of the source package
+    """
     try:
         srcrecords = apt.apt_pkg.SourceRecords()
         while srcrecords.lookup(srcname):
@@ -567,6 +894,23 @@ def get_source_version(srcname):
 
 
 def get_source_package(package, only_source=False):
+    """
+    Get list of binary packages belonging to a (source) package.
+
+    Parameters
+    ----------
+    package : str
+        the name of a (source or binary) package
+    only_source : bool
+        should be set to True if only source package names should be
+        matched (like apt-cache showsrc --only-source)
+
+    Returns
+    -------
+    [(str, str), ...]
+        A list of (name, short description) tuples describing the binary
+        packages belonging to the source package of the given package
+    """
     packages = []
     found = set()
     try:
@@ -598,6 +942,36 @@ def get_source_package(package, only_source=False):
 
 
 def get_package_info(packages, skip_notfound=False):
+    """
+    Collect information about one or more packages
+
+    This is a helper function for :func:`packages_providing()` and
+    :func:`get_dependency_info()`.
+
+    Parameters
+    ----------
+    packages : [((str, ...), str), ...]
+        List of ((pkg1, pkg2, ...), package) tuples about which to look
+        up information. Each tuple consists of two elements:
+
+        - another tuple specifying a "group" of package names
+          representing alternative dependencies
+        - one specific package name that should also be part of the
+          group. Virtual package names are allowed.
+
+    skip_notfound : bool
+        Include information about (group, package) tuples about which no
+        information at all was found, indicating just this.
+
+    Returns
+    -------
+    [(str, str, str, str, str or None), ...]
+        List of (name, status, version, shortdescription, provides)
+        tuples about each specific package in the input tuples (not
+        groups). If multiple packages providing one of the packages are
+        found, information on all of these packages is returned and the
+        corresponding "provides" field is filled in the output tuples.
+    """
     if not packages:
         return []
 
@@ -676,6 +1050,21 @@ def get_package_info(packages, skip_notfound=False):
 
 
 def packages_providing(package):
+    """
+    Get a list of packages providing a given package name
+
+    Parameters
+    ----------
+    package : str
+        package name to be looked up in 'Package:' and 'Provides:'
+        fields
+
+    Returns
+    -------
+    [(str, str), ...]
+        List of (name, shortdescription) tuples for each (non-virtual)
+        package providing the given (possibly virtual) package
+    """
     aret = get_package_info([((package,), package)], skip_notfound=True)
     ret = []
     for pkg in aret:
@@ -685,6 +1074,30 @@ def packages_providing(package):
 
 
 def get_dependency_info(package, depends, rel="depends on"):
+    """
+    Provide information on the dependencies of a package
+
+    For each of the provided dependencies, the status and version is
+    collected. The result is formatted as a table and returned as a
+    single string.
+
+    Parameters
+    ----------
+    package : str
+        name of the package to which the dependency list belongs
+    depends : [(str, ...), (str, ...), ...]
+        list of tuples, each tuple giving a "dependency group" of simple
+        dependencies or groups of alternative dependencies
+    rel : str
+        should be either "depends on", "recommends", or "suggests" and
+        will be printed along with the package name above the dependency
+        table
+
+    Returns
+    -------
+    str
+        a table listing the status and versions of dependencies
+    """
     if not depends:
         return ('\n%s %s no packages.\n' % (package, rel))
 
@@ -739,12 +1152,33 @@ def get_dependency_info(package, depends, rel="depends on"):
 
 
 def get_changed_config_files(conffiles, nocompress=False):
+    """
+    Get information about modified config files
+
+    Parameters
+    ----------
+    conffiles : [(str, str), ...]
+        List of (filename, md5sum) tuples
+    nocompress : bool
+        determines whether empty lines and comment lines should be
+        included in the output
+
+    Returns
+    -------
+    ({str: str, ... }, [str, ...])
+        tuple with two components:
+
+        - dictionary of modified config files, with filenames as keys
+          and ("compressed" if desired) config file content as values
+        - filename list of the modified config files
+    """
     confinfo = {}
     changed = []
     for (filename, md5sum) in conffiles:
         try:
-            fp = open(filename, errors='backslashreplace')
-        except IOError as msg:
+            with open(filename, errors='backslashreplace') as fp:
+                lines = fp.readlines()
+        except OSError as msg:
             confinfo[filename] = msg
             continue
 
@@ -754,7 +1188,7 @@ def get_changed_config_files(conffiles, nocompress=False):
 
         changed.append(filename)
         thisinfo = 'changed:\n'
-        for line in fp:
+        for line in lines:
             if not line:
                 continue
 
@@ -774,6 +1208,17 @@ DISTORDER = ['oldstable', 'stable', 'testing', 'unstable', 'experimental']
 
 
 def get_debian_release_info():
+    """
+    Collect information about the distribution and release
+
+    Find out which distribution (Debian/Ubuntu/) this is, which release,
+    and APT preferences and policy.
+
+    Returns
+    -------
+    str
+        text summarizing the collected information
+    """
     debvers = debinfo = verfile = warn = ''
     dists = []
     output = get_command_output('apt-cache policy 2>/dev/null')
@@ -800,7 +1245,7 @@ def get_debian_release_info():
     try:
         with open('/etc/debian_version', errors='backslashreplace') as fob:
             verfile = fob.readline().strip()
-    except IOError:
+    except OSError:
         print('Unable to open /etc/debian_version', file=sys.stderr)
 
     if verfile:
@@ -819,16 +1264,25 @@ def get_debian_release_info():
 
 
 def lsb_release_info():
+    """
+    Get LSB release information
+
+    Returns
+    -------
+    str
+        Information about LSB release
+    """
     return get_command_output('lsb_release -a 2>/dev/null')
 
 
 def get_arch():
-    """Get the architecture of the current system.
-
-    :returns:
-
-        The architecture, e.g. ``"i386"``.
+    """
+    Get the architecture of the current system.
 
+    Returns
+    -------
+    str
+        architecture name (e.g., ``"i386"``)
     """
     arch = get_command_output('COLUMNS=79 dpkg --print-architecture 2>/dev/null').strip()
     if not arch:
@@ -841,6 +1295,14 @@ def get_arch():
 
 
 def get_multiarch():
+    """
+    Check if multiarch is used (foreign architecture(s) defined)
+
+    Returns
+    -------
+    str
+        comma-separated list of foreign architectures
+    """
     out = get_command_output('COLUMNS=79 dpkg --print-foreign-architectures 2>/dev/null')
     return ', '.join(out.splitlines())
 
@@ -851,6 +1313,57 @@ def generate_blank_report(package, pkgversion, severity, justification,
                           subject='', tags='', body='', mode=MODE_EXPERT,  # noqa: F821
                           pseudos=None, debsumsoutput=None, issource=False,
                           options=None):
+    """
+    Generate a bug report template
+
+    Parameters
+    ----------
+    package : str
+        package name
+    pkgversion: str
+        package version
+    severity : str
+        bug severity
+    justification : str
+        reason why critical severity is justified (if applicable)
+    depinfo : str
+        dependency information (like from
+        :func:`get_dependency_info():func:`)
+    confinfo : str
+        config file information
+    foundfile : str, optional
+        the name of the file where the bug is
+    incfiles : str, optional
+        files to include in the report
+    system : str, optional
+        most often "debian"
+    exinfo : int, debianbts.Bugreport or False
+        number of the bug for which this message provides extra
+        information
+    type : str, optional
+        should be either 'debbugs' or 'launchpad'
+    klass : str, optional
+        unused
+    subject : str, optional
+        could be bug report mail subject
+    tags : str, optional
+        tags to be set for this bug
+    body : str, optional
+        main report message text
+    mode : int, optional
+        reportbug mode (novice, expert, etc.)
+    pseudos : str, optional
+        pseudo-headers to include inline in the report body
+    debsumsoutput : str, optional
+        debsums error output text
+    issource : bool, optional
+        flag whether this report is on a source or binary package
+
+    Returns
+    -------
+    str
+        bug report message template with inline system information
+    """
     # For now...
     from . import bugreport
 
@@ -869,7 +1382,9 @@ def generate_blank_report(package, pkgversion, severity, justification,
 
 
 class our_lex(shlex.shlex):
+    """Quote-removing lexical analyzer object"""
     def get_token(self):
+        """Get token by token, with quotes removed"""
         token = shlex.shlex.get_token(self)
         if token is None or not len(token):
             return token
@@ -892,16 +1407,35 @@ CONFIG_ARGS = (
 
 
 def first_run():
+    """
+    Check if user configuration file exists
+
+    Unless reportbug is run for the first time, the user config file
+    will often exist.
+
+    Returns
+    -------
+    bool
+        True if reportbug is run for the first time
+    """
     return not os.path.exists(USERFILE)
 
 
 def parse_config_files():
+    """
+    Parse the reportbug configuration files
+
+    Returns
+    -------
+    dict
+        dictionary with configuration parameter and their settings
+    """
     args = {}
     for filename in FILES:
         if os.path.exists(filename):
             try:
                 lex = our_lex(open(filename, errors="backslashreplace"), posix=True)
-            except IOError:
+            except OSError:
                 continue
 
             lex.wordchars = lex.wordchars + '-.@/:<>'
@@ -997,11 +1531,26 @@ def parse_config_files():
 
 
 def parse_bug_control_file(filename):
+    """
+    Parse a packages bug control file
+
+    Parameters
+    ----------
+    filename : str
+        bug control file name
+
+    Returns
+    -------
+    (str, str, [str, ...] , [str, ...])
+        tuple with (submit-as, send-to, report-with, package-status)
+        control field values
+    """
     submitas = submitto = None
     reportwith = []
     supplemental = []
-    fh = open(filename, errors='backslashreplace')
-    for line in fh:
+    with open(filename, errors='backslashreplace') as fp:
+        lines = fp.readlines()
+    for line in lines:
         line = line.strip()
         parts = line.split(': ')
         if len(parts) != 2:
@@ -1021,6 +1570,50 @@ def parse_bug_control_file(filename):
 
 
 def cleanup_msg(dmessage, headers, pseudos, btstype):
+    """
+    Clean up the bug report message
+
+    Cleaning up consists of splitting off the headers and pseudo-headers
+    from the message body. Two variants are distinguished, determined by
+    the value of parameter `btstype`:
+
+    - In "debbugs"-mode, pseudo-headers are recognized based on an
+      internal list of known pseudoheader fields. This list can be
+      extended using the `pseudos` parameter. Cleaning up includes
+      making sure that only a single instance of non-repeatable
+      pseudo-headers is present. If multiple instances are present in
+      the input, the last one is used.
+
+    - In "gnats"-mode, pseudo-headers start with the character '>'; the
+      `pseudos` parameter is ignored.
+
+    Headers not recognized as known pseudoheader fields are assumed to
+    be proper headers.
+
+    Parameters
+    ----------
+    dmessage : str
+        raw message. Headers and pseudoheaders found in the message will
+        be split off and returned separately.
+    headers : [str, ...]
+        list of header strings. Headers and header content provided here
+        are passed to the output.
+    pseudos : [str, ...]
+        list of additional fields to recognize as pseudoheaders. The
+        field strings must include a colon; any further content after
+        the colon is discarded. Field names must be capitalized.
+    btstype : str
+        should be either "debbugs" or "gnats"
+
+    Returns
+    -------
+    (str, [(str, str), ...], [str, ...])
+        tuple with three components:
+
+        - "cleaned" message string
+        - list of (header, content) tuples representing headers
+        - list of pseudoheaders
+    """
     newheaders = []
     collected_pseudoheaders = []
     clean_pseudoheaders = []
@@ -1122,9 +1715,28 @@ def cleanup_msg(dmessage, headers, pseudos, btstype):
 
 
 def launch_mbox_reader(cmd, url, http_proxy, timeout):
-    """Runs the command specified by cmd passing the mbox file
+    """
+    Download an mbox and open it in a mail user agent.
+
+    Runs the command specified by cmd, passing the mbox file
     downloaded from url as a parameter. If cmd is None or fails, then
-    fallback to mail program."""
+    fallback to mail program.
+
+    Parameters
+    ----------
+    cmd : str
+        command to open the mbox file with, 'mail -f' if None
+    url : str
+        URL to download the mbox file from
+    http_proxy : str
+        http proxy
+    timeout : int
+        download connection timeout in seconds
+
+    Returns
+    -------
+    None
+    """
     mbox = open_url(url, http_proxy, timeout)
     if mbox is None:
         return
@@ -1148,9 +1760,18 @@ def launch_mbox_reader(cmd, url, http_proxy, timeout):
 
 
 def get_running_kernel_pkg():
-    """Return the package of the currently running kernel, needed to force
-    assignment for 'kernel' package to a real one"""
+    """
+    Get the package of the currently running kernel
 
+    Identifying the running kernel is needed to force
+    assignment for 'kernel' package to a real one.
+
+    Returns
+    -------
+    str or None
+        the name of the kernel package, unless it could not be
+        determined
+    """
     system = platform.system()
     release = platform.release()
 
@@ -1163,9 +1784,29 @@ def get_running_kernel_pkg():
 
 
 def exec_and_parse_bugscript(handler, bugscript, runner=os.system):
-    """Execute and parse the output of the package bugscript, in particular
-    identifying the headers and pseudo-headers blocks, if present"""
+    """
+    Execute and parse the output of the package bugscript
 
+    Package bugscripts generate output that can be split into the
+    following sections: headers, pseudo-headers, text, and a list of
+    attachments. All sections are optional.
+
+    This function executes a package bugscript and parses the ouput,
+    identifying the section blocks, if present.
+
+    Parameters
+    ----------
+    handler : str
+        name of an executable
+    bugscript : str
+        bugscript filename
+
+    Returns
+    -------
+    (int, str, str, [str, ...])
+        Tuple with 5 components: (returncode, headers, pseudoheaders,
+        remaining text, list of attachments)
+    """
     fh, filename = TempFile()
     fh.close()
     rc = runner('LC_ALL=C %s %s %s' % (handler, shlex.quote(bugscript),
@@ -1176,8 +1817,9 @@ def exec_and_parse_bugscript(handler, bugscript, runner=os.system):
     isattachments = False
     headers = pseudoheaders = text = ''
     attachments = []
-    fp = open(filename, errors="backslashreplace")
-    for line in fp.readlines():
+    with open(filename, errors="backslashreplace") as fp:
+        lines = fp.readlines()
+    for line in lines:
         # we identify the blocks for headers and pseudo-h
         if line == '-- BEGIN HEADERS --\n':
             isheaders = True
@@ -1200,26 +1842,37 @@ def exec_and_parse_bugscript(handler, bugscript, runner=os.system):
                 attachments.append(line.strip())
             else:
                 text += line
-    fp.close()
     cleanup_temp_file(filename)
 
     return (rc, headers, pseudoheaders, text, attachments)
 
 
 def check_package_name(pkg):
-    """Check the package name against Debian Policy:
+    """
+    Check the package name against Debian Policy
+
+    Package names must comply with the policy requirements:
     https://www.debian.org/doc/debian-policy/ch-controlfields.html#s-f-Source
 
-    Returns True if the package name is valid."""
-
+    Returns
+    -------
+    bool
+        True if the package name is valid.
+    """
     pkg_re = re.compile(r'^[a-z0-9][a-z0-9+-\.]+$')
 
     return True if pkg_re.match(pkg) else False
 
 
 def get_init_system():
-    """Determines the init system on the current machine"""
+    """
+    Determines the init system on the current machine
 
+    Returns
+    -------
+    str
+        the name of the init system, and how it was determined
+    """
     init = 'unable to detect'
 
     if os.path.isdir('/run/systemd/system'):
@@ -1242,11 +1895,15 @@ def get_init_system():
 
 
 def get_lsm_info():
-    """Determines the linux security module enabled on the current machine
-
-    Returns None if there is no LSM enabled on the machine or if the state
-    cannot be determined."""
+    """
+    Determines the linux security module enabled on the current machine
 
+    Returns
+    -------
+    str or None
+        Description of LSM info; None if there is no LSM enabled on the
+        machine or if the state cannot be determined
+    """
     lsminfo = None
 
     if os.path.exists('/usr/bin/aa-enabled') \
@@ -1271,8 +1928,13 @@ def get_lsm_info():
 
 
 def get_kernel_taint_flags():
-    """Determines the kernel taint flags"""
+    """
+    Determines the kernel taint flags
 
+    Returns
+    -------
+    list of strings
+    """
     # https://github.com/torvalds/linux/blob/cedc5b6aab493f6b1b1d381dccc0cc082da7d3d8/include/linux/kernel.h#L582
     # this is going to need updating (but maybe not that often)
     TAINT_FLAGS = [
@@ -1299,7 +1961,8 @@ def get_kernel_taint_flags():
     flags = []
 
     if os.path.exists('/proc/sys/kernel/tainted'):
-        tainted = int(open('/proc/sys/kernel/tainted').read())
+        with open('/proc/sys/kernel/tainted') as fp:
+            tainted = int(fp.read())
 
         # tainted is an integer representing a bitmask, so logical-AND against the list of
         # flags and if it's a TRUE, then append it to the list of flags enabled
@@ -1311,7 +1974,8 @@ def get_kernel_taint_flags():
 
 
 def is_security_update(pkgname, pkgversion):
-    """Determine whether a given package is a security update.
+    """
+    Determine whether a given package is a security update
 
     Detection of security update versions works most reliably if the
     package version under investigation is the currently installed
@@ -1331,7 +1995,6 @@ def is_security_update(pkgname, pkgversion):
         True if there is evidence that this version is a security
         update, otherwise False
     """
-
     # Check 1:
     # If it does not follow the debXuY version number pattern, it is
     # definitely no security update.
